@@ -1,6 +1,6 @@
 //use ruma_identifiers::{RoomId, UserId};
 //use serde::{Serialize, Deserialize};
-use crackle_lib::{RoomId, RoomName, UserId};
+use crackle_lib::{RoomAliasId, RoomId, RoomName, UserId};
 use crossbeam_channel::{unbounded, TryRecvError};
 use crossterm::{
     event::{self, DisableMouseCapture, EnableMouseCapture, Event, KeyCode},
@@ -37,6 +37,7 @@ struct CrackleConfig {
 
 enum Focus {
     MessageInput,
+    JoinRoomInput,
     //RoomList,
     //PaticipantList,
     //History,
@@ -64,8 +65,9 @@ struct App {
     rooms: BTreeMap<RoomId, Box<RoomName>>,
     current_room: Option<RoomId>,
     //using_config: bool,
+    room_id_to_canonical_alias: BTreeMap<RoomId, RoomAliasId>,
     auth_token: Option<String>,
-    //message_box_height: u16,
+    joined_rooms: Vec<RoomId>, //message_box_height: u16,
 }
 /*
 impl Default for App {
@@ -116,6 +118,8 @@ impl App {
             selected_room: None,
             current_room: None,
             rooms: BTreeMap::new(),
+            room_id_to_canonical_alias: BTreeMap::new(),
+            joined_rooms: Vec::new(),
             auth_token: auth_token,
             room: room,
             //message_box_height: 3,
@@ -204,26 +208,16 @@ enum SyncMessage {
     Quit,
     //Response(sync::sync_events::v3::Response),
     SendMessage(RoomId, String, String),
-    JoinedRooms(BTreeMap<RoomId, Box<RoomName>>),
+    RoomsUpdated(
+        Vec<RoomId>,
+        BTreeMap<RoomId, Box<RoomName>>,
+        BTreeMap<RoomId, RoomAliasId>,
+    ),
+    LeaveRoom(RoomId),
+    JoinRoom(String),
 }
 #[allow(unused_assignments)]
 async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Result<()> {
-    /*
-    let mut client = ruma_client::Client::builder().homeserver_url("https://sonicrules.fun:8448/".parse().unwrap()).build::<ruma_client::http_client::Reqwest>().await.unwrap();//.access_token(Some(std::fs::read_to_string("auth_token.txt").unwrap().trim().to_string())).build();
-    let mut login_result = client.log_in("@crackle_client:sonicrules.fun", "50nicru1e5", None, None).await;
-    while login_result.is_err() {
-        login_result = client.log_in("@crackle_client:sonicrules.fun", "50nicru1e5", None, None).await;
-    }
-    //let mut sync_request = sync::sync_events::v3::Request::new();
-    //sync_request.full_state = true;
-    //let mut sync_result = client.send_request(sync_request.clone()).await;
-    //while sync_result.is_err() {
-    //    sync_result = client.send_request(sync_request.clone()).await;
-    //}
-    //let sync_response: sync::sync_events::v3::Response = sync_result.unwrap();
-    */
-    //let distrotube_room_id = client.send_request(ruma_client_api::alias::get_alias::v3::Request::new(ruma_identifiers::room_alias_id!("#distrotube:matrix.org"))).await.unwrap().room_id;
-    //println!("'{}'", app.server_name.clone());
     let mut client = crackle_lib::Client::new(
         app.host.clone(),
         app.user_name.clone(),
@@ -259,6 +253,12 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
                 Ok(SyncMessage::SendMessage(room_id, _user_id, message)) => {
                     client.send_message(room_id.try_into().unwrap(), message);
                 }
+                Ok(SyncMessage::LeaveRoom(room_id)) => {
+                    client.leave_room_id(room_id);
+                }
+                Ok(SyncMessage::JoinRoom(room)) => {
+                    client.join_with_alias(room);
+                }
                 Ok(_) => {}
                 Err(TryRecvError::Empty) => {}
                 Err(_) => break,
@@ -266,8 +266,13 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
             sync_results = client.sync().unwrap();
             let messages = client.extract_data(sync_results.clone()); //, client.clone().me);
             if client.room_list_updated() {
-                tx1.send(SyncMessage::JoinedRooms(client.data.room_to_name.clone()))
-                    .unwrap();
+                //println!("{:#?}", client.data.joined_rooms);
+                tx1.send(SyncMessage::RoomsUpdated(
+                    client.data.joined_rooms.clone(),
+                    client.data.room_to_name.clone(),
+                    client.data.room_id_to_cananoical_alias.clone(),
+                ))
+                .unwrap();
             }
             for message in messages {
                 let room_id = message.room_id;
@@ -286,26 +291,25 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
             if let Event::Key(key) = event::read()? {
                 match app.focus {
                     Focus::RoomSelect => {
-                        let num_rooms = app.rooms.len();
-                        if num_rooms > 1 {
-                            /*
-                            let mut prev: RoomId;
-                            let mut next: RoomId;
-                            let last = app.rooms.iter().last().unwrap().0;
-                            let first: RoomId;
-                            for (index_num, (room_id, _room_name)) in app.rooms.iter().enumerate() {
-                                if index_num == 0 {
-                                    first = room_id;
-                                }
+                        //let num_rooms = app.rooms.len();
+                        /*
+                        let mut prev: RoomId;
+                        let mut next: RoomId;
+                        let last = app.rooms.iter().last().unwrap().0;
+                        let first: RoomId;
+                        for (index_num, (room_id, _room_name)) in app.rooms.iter().enumerate() {
+                            if index_num == 0 {
+                                first = room_id;
                             }
-                            */
-                            match key.code {
-                                KeyCode::Up => {
-                                    let last = app.rooms.iter().last().unwrap().0;
-                                    let first = app.rooms.iter().nth(0).unwrap().0;
+                        }
+                        */
+                        match key.code {
+                            KeyCode::Up => {
+                                if app.joined_rooms.len() > 1 {
+                                    let last = app.joined_rooms.iter().last().unwrap();
+                                    let first = app.joined_rooms.iter().nth(0).unwrap();
                                     let mut prev = last.clone();
-                                    for (index_num, (room_id, _room_name)) in
-                                        app.rooms.iter().enumerate()
+                                    for (index_num, room_id) in app.joined_rooms.iter().enumerate()
                                     {
                                         if index_num == 0 {
                                             if first.clone() == app.selected_room.clone().unwrap() {
@@ -325,11 +329,13 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
                                           //}
                                     }
                                 }
-                                KeyCode::Down => {
-                                    let last = app.rooms.iter().last().unwrap().0;
-                                    let first = app.rooms.iter().nth(0).unwrap().0;
+                            }
+                            KeyCode::Down => {
+                                if app.joined_rooms.len() > 1 {
+                                    let last = app.joined_rooms.iter().last().unwrap();
+                                    let first = app.joined_rooms.iter().nth(0).unwrap();
                                     let mut use_next = false;
-                                    for (room_id, _room_name) in app.rooms.iter() {
+                                    for room_id in app.joined_rooms.iter() {
                                         if use_next {
                                             app.selected_room = Some(room_id.clone());
                                             break;
@@ -350,17 +356,29 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
                                         }
                                     }
                                 }
-                                KeyCode::Enter => {
-                                    app.current_room = Some(app.selected_room.clone().unwrap());
-                                    app.selected_room = None;
-                                    app.focus = Focus::Watching;
-                                }
-                                KeyCode::Esc => {
-                                    app.selected_room = None;
-                                    app.focus = Focus::Watching;
-                                }
-                                _ => {}
                             }
+                            KeyCode::Char('l') => {
+                                tx2.send(SyncMessage::LeaveRoom(
+                                    app.selected_room.clone().unwrap(),
+                                ))
+                                .unwrap();
+                                //app.rooms.remove(app.selected_room.as_ref().unwrap());
+                                app.selected_room = None;
+                                //app.current_room = None;
+                            }
+                            KeyCode::Char('j') => {
+                                app.focus = Focus::JoinRoomInput;
+                            }
+                            KeyCode::Enter => {
+                                app.current_room = Some(app.selected_room.clone().unwrap());
+                                app.selected_room = None;
+                                app.focus = Focus::Watching;
+                            }
+                            KeyCode::Esc => {
+                                app.selected_room = None;
+                                app.focus = Focus::Watching;
+                            }
+                            _ => {}
                         }
                     }
                     Focus::Watching => match key.code {
@@ -390,13 +408,43 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
                         KeyCode::Enter => {
                             let message: String = app.input.drain(..).collect();
                             //app.messages.push(message.clone());
-                            let sync_message = SyncMessage::SendMessage(
-                                app.current_room.clone().unwrap(),
-                                format!("{}", me),
-                                message,
-                            );
-                            tx1_clone.send(sync_message.clone()).unwrap();
+                            if message != String::new() {
+                                let sync_message = SyncMessage::SendMessage(
+                                    app.current_room.clone().unwrap(),
+                                    format!("{}", me),
+                                    message,
+                                );
+                                tx1_clone.send(sync_message.clone()).unwrap();
+                                tx2.send(sync_message.clone()).unwrap();
+                            }
+                        }
+                        KeyCode::Char(c) => {
+                            app.input.push(c);
+                        }
+                        KeyCode::Backspace => {
+                            app.input.pop();
+                        }
+                        KeyCode::Esc => {
+                            app.focus = Focus::Watching;
+                        }
+                        _ => {
+                            /*
+                            if let Ok(msg) = rx1.try_recv() {
+                                if let SyncMessage::SendMessage(room_id, user_id, message) = msg {
+                                    app.messages.push(format!("[{}] <{}> {}", room_id, user_id, message));
+                                }
+                            }
+                            */
+                        }
+                    },
+                    Focus::JoinRoomInput => match key.code {
+                        KeyCode::Enter => {
+                            let room: String = app.input.drain(..).collect();
+                            //app.messages.push(message.clone());
+                            let sync_message = SyncMessage::JoinRoom(room);
+                            //tx1_clone.send(sync_message.clone()).unwrap();
                             tx2.send(sync_message.clone()).unwrap();
+                            app.focus = Focus::Watching;
                         }
                         KeyCode::Char(c) => {
                             app.input.push(c);
@@ -427,17 +475,33 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
                         msgs.remove(0);
                     }
                     msgs.push(format!("<{}> {}", user_id, message));
-                    app.messages.insert(room_id.clone(), msgs).unwrap();
-                } else if let SyncMessage::JoinedRooms(room_id_list) = msg {
+                    app.messages.insert(room_id.clone(), msgs);
+                } else if let SyncMessage::RoomsUpdated(
+                    room_id_list,
+                    room_to_name,
+                    room_id_to_canonical_alias,
+                ) = msg
+                {
+                    //app.rooms = room_id_list;
+                    app.joined_rooms = room_id_list.clone();
+                    app.rooms = room_to_name;
+                    app.room_id_to_canonical_alias = room_id_to_canonical_alias;
+                    for room_id in room_id_list {
+                        if !app.messages.contains_key(&room_id) {
+                            app.messages.insert(room_id.clone(), Vec::new());
+                        }
+                    }
+                    /*
                     for (room_id, room_name) in room_id_list {
                         if app.current_room.is_none() {
                             app.current_room = Some(room_id.clone());
                         }
                         if !app.rooms.contains_key(&room_id.clone()) {
-                            app.rooms.insert(room_id.clone(), room_name.clone());
+                            app.rooms.aaaaaa(room_id.clone(), room_name.clone());
                             app.messages.insert(room_id.clone(), Vec::new());
                         }
                     }
+                    */
                 }
             }
         }
@@ -485,13 +549,23 @@ fn ui<B: Backend>(f: &mut Frame<B>, app: &mut App) {
             ],
             Style::default(),
         ),
+        Focus::JoinRoomInput => (
+            vec![
+                Span::raw("Press "),
+                Span::styled("Esc", Style::default().add_modifier(Modifier::BOLD)),
+                Span::raw(" to stop editing, "),
+                Span::styled("Enter", Style::default().add_modifier(Modifier::BOLD)),
+                Span::raw(" to join the room"),
+            ],
+            Style::default(),
+        ),
         Focus::RoomSelect => (
             vec![
                 Span::raw("Press "),
                 Span::styled("Esc", Style::default().add_modifier(Modifier::BOLD)),
                 Span::raw(" to stop choosing a room with the Up and Down arrow keys, "),
                 Span::styled("Enter", Style::default().add_modifier(Modifier::BOLD)),
-                Span::raw(" to choose the selected room"),
+                Span::raw(" to view the selected room.  You can press l to leave the room or j to join a new one"),
             ],
             Style::default(),
         ),
@@ -506,6 +580,7 @@ fn ui<B: Backend>(f: &mut Frame<B>, app: &mut App) {
             Focus::Watching => Style::default(),
             Focus::MessageInput => Style::default().fg(Color::Yellow),
             Focus::RoomSelect => Style::default(),
+            Focus::JoinRoomInput => Style::default().fg(Color::Yellow),
         })
         .block(Block::default().borders(Borders::ALL).title("Input"));
     match app.focus {
@@ -522,14 +597,32 @@ fn ui<B: Backend>(f: &mut Frame<B>, app: &mut App) {
                 chunks[2].y + 1,
             )
         }
+        Focus::JoinRoomInput => {
+            // Make the cursor visible and ask tui-rs to put it at the specified coordinates after rendering
+            f.set_cursor(
+                // Put cursor past the end of the input text
+                chunks[2].x + app.input.width() as u16 + 1,
+                // Move one line down, from the border to the input line
+                chunks[2].y + 1,
+            )
+        }
         Focus::RoomSelect => {}
     }
+    //println!("{:#?}", app.joined_rooms);
     let room_list: Vec<ListItem> = app
-        .rooms
+        .joined_rooms
         .iter()
         //.enumerate()
-        .map(|(room_id, room_name)| {
-            let content = vec![Spans::from(Span::raw(format!("{}", room_name)))];
+        .map(|room_id| {
+            let content: Vec<Spans>;
+            if app.rooms.contains_key(&room_id) {
+                content = vec![Spans::from(Span::raw(format!("{}", app.rooms[room_id])))];
+            } else {
+                content = vec![Spans::from(Span::raw(format!(
+                    "{}",
+                    app.room_id_to_canonical_alias[room_id]
+                )))];
+            }
             //let content: Vec<Span>;
             if app.current_room.is_none() {
                 app.current_room = Some(room_id.clone());
